package net.phoenixinquis.android.juiz;

import java.util.Calendar;
import android.os.Handler;
import android.os.SystemClock;
import android.content.Context;
import android.content.res.Resources;
import android.view.SurfaceHolder;
import android.view.WindowManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.DisplayMetrics;
import android.service.wallpaper.WallpaperService;
import android.service.wallpaper.WallpaperService.Engine;

public class JuizWallpaper extends WallpaperService {
    private final Handler mHandler = new Handler();
    
    @Override
    public Engine onCreateEngine() {
        return new JuizEngine();
    }
    class JuizEngine extends Engine {
        private final Paint mPaint = new Paint();
        private boolean mVisible;
        private long mStartTime;
        private int mStatusBarHeight;
        private int mWidth;
        private int mHeight;
        private Bitmap mLogo;
        private Bitmap mNoblesse;
        private Bitmap mSelecao;
        
        private final Runnable mDrawWallpaper = new Runnable() {
            public void run() {
                drawFrame();
            }
        };
        
        JuizEngine() {
            final Paint paint = mPaint;
            paint.setColor(0xffff4e5d);
            mStartTime = SystemClock.elapsedRealtime();
            Resources res = getResources();
            mLogo = BitmapFactory.decodeResource(res, R.drawable.noblesse_logo);
            mNoblesse = BitmapFactory.decodeResource(res, R.drawable.noblesse_oblige);
            BitmapFactory.Options options = new BitmapFactory.Options();
            options.inScaled = false;
            mSelecao = BitmapFactory.decodeResource(res, R.drawable.selecao, options);
        }
        
        @Override
        public void onVisibilityChanged(boolean visible) {
            mVisible = visible;
            if (visible) {
                drawFrame();
            } else {
                mHandler.removeCallbacks(mDrawWallpaper);
            }
        }

        @Override
        public void onSurfaceChanged(SurfaceHolder holder, int format, int width, int height) {
            super.onSurfaceChanged(holder, format, width, height);
            DisplayMetrics displayMetrics = new DisplayMetrics();
            ((WindowManager) getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay().getMetrics(displayMetrics);
            switch (displayMetrics.densityDpi) {
                case DisplayMetrics.DENSITY_HIGH:
                    mStatusBarHeight = 38;
                    break;
                case DisplayMetrics.DENSITY_LOW:
                    mStatusBarHeight = 19;
                    break;
                case DisplayMetrics.DENSITY_MEDIUM:
                default:
                    mStatusBarHeight = 25;
            }
            mWidth = width;
            mHeight = height;
            drawFrame();
        }
        void drawFrame() {
            final SurfaceHolder holder = getSurfaceHolder();

            Canvas c = null;
            try {
                c = holder.lockCanvas();
                if (c != null) {
                    c.drawColor(0xff000000);
                    // draw something
                    mPaint.setAlpha(0xaa);
                    DrawLogo(c);
                    DrawNoblesse(c);
                    mPaint.setAlpha(0xff);
                    DrawNotificationArea(c);
                    DrawSelecao(c);
                }
            } finally {
                if (c != null) holder.unlockCanvasAndPost(c);
            }

            // Reschedule the next redraw
            mHandler.removeCallbacks(mDrawWallpaper);
            if (mVisible) {
                mHandler.postDelayed(mDrawWallpaper, 1000 / 25);
            }
        }
        
        void DrawLogo(Canvas c) {
            c.drawBitmap(mLogo, 180, mHeight - 330, mPaint);
        }
        void DrawNoblesse(Canvas c) {
            c.save(Canvas.CLIP_SAVE_FLAG);
            int x = mWidth - 100;
            int y = 240 + mStatusBarHeight;
            int w = mNoblesse.getWidth();
            int h = mHeight - y;
            long now = SystemClock.elapsedRealtime();
            long offset = mNoblesse.getHeight() - (((now - mStartTime) / 7) % mNoblesse.getHeight());
            mPaint.setStyle(Paint.Style.FILL);
            Rect clip = new Rect(x, y, x + w, y + h);
            c.clipRect(clip);
            long top = y + h + mNoblesse.getHeight() - offset;
            while (top > y - mNoblesse.getHeight()) {
                c.drawBitmap(mNoblesse, x, top, mPaint);
                top -= mNoblesse.getHeight();
            }
            c.drawBitmap(mNoblesse, x - w, y - h, mPaint);
            c.restore();
        }
        void DrawNotificationArea(Canvas c) {
            float lineWidth = 2;
            float lineSpacing = 5;
            mPaint.setStyle(Paint.Style.STROKE);
            mPaint.setStrokeWidth(lineWidth);
            float x = mWidth - 50;
            for (int i = 0; i < 7; i++) {
                c.drawLine(x, 240 + mStatusBarHeight, x, mHeight - 100, mPaint);
                x += (lineWidth + lineSpacing);
            }
        }
        void DrawSelecao(Canvas c) {
            int iconHeight = mSelecao.getHeight() / 4;
            int iconWidth = mSelecao.getWidth() / 6;
            mPaint.setStyle(Paint.Style.STROKE);
            mPaint.setStrokeWidth(2);
            mPaint.setAlpha(0xaa);
            Calendar cal = Calendar.getInstance();
            for (int i = 0; i < 12; i++) {
                /* The Selecao icons will be lit to display the current hour of the day.
                 * 
                 * In the AM, an additional icon will be lit each hour until all are lit at noon.
                 * In the PM, each icon will darken each hour until all are out at midnight.
                 */
                
                int hour = cal.get(cal.HOUR_OF_DAY);
                boolean offline = !((hour % 12) > i);
                if (hour >= 12) offline = !offline;
                int x = (i % 6) * iconWidth;
                int y = (i / 6) * iconHeight;
                Rect src = new Rect(x, y, x + iconWidth, y + iconHeight);
                if (offline) {
                    src.top += 236;
                    src.bottom += 236;
                    src.left += 2;
                    src.right += 2;
                }
                Rect dst = new Rect(x + 11, y + mStatusBarHeight, x + iconWidth + 11, y + iconHeight + mStatusBarHeight);
                c.drawBitmap(mSelecao, src, dst, mPaint);
            }
            mPaint.setAlpha(0xff);
            c.drawRect(10, mStatusBarHeight + 2, mSelecao.getWidth() + 10, (iconHeight * 2) + mStatusBarHeight, mPaint);
        }
    }
}
